"""The model hub is a central repository of models that can be used to load pretrained models."""


def load_model_from_hub(model_name: str, **kwargs):
    """Load a model from the model hub.

    :param model_name: The name of the model to load.
    :param kwargs: Additional arguments to pass to the model.
    :return: The model.
    """
    ...
